@@ -23,7 +23,6 @@
 #include <QColor>
 #include <memory>
 
-#include "qgsabstractgeometry.h"
 #include "qgscoordinatetransform.h"
 #include "qgsexpressioncontext.h"
 #include "qgsfeaturefilterprovider.h"
@@ -39,6 +38,7 @@ class QPainter;
 class QgsAbstractGeometry;
 class QgsLabelingEngine;
 class QgsMapSettings;
+class QgsRenderedFeatureHandlerInterface;
 
 
 /**
@@ -72,9 +72,54 @@ class CORE_EXPORT QgsRenderContext
       Antialiasing             = 0x80,  //!< Use antialiasing while drawing
       RenderPartialOutput      = 0x100, //!< Whether to make extra effort to update map image with partially rendered layers (better for interactive map canvas). Added in QGIS 3.0
       RenderPreviewJob         = 0x200, //!< Render is a 'canvas preview' render, and shortcuts should be taken to ensure fast rendering
+      RenderBlocking           = 0x400, //!< Render and load remote sources in the same thread to ensure rendering remote sources (svg and images). WARNING: this flag must NEVER be used from GUI based applications (like the main QGIS application) or crashes will result. Only for use in external scripts or QGIS server.
     };
     Q_DECLARE_FLAGS( Flags, Flag )
 
+    /**
+     * Options for rendering text.
+     * \since QGIS 3.4.3
+     */
+    enum TextRenderFormat
+    {
+      // refs for below dox: https://github.com/qgis/QGIS/pull/1286#issuecomment-39806854
+      // https://github.com/qgis/QGIS/pull/8573#issuecomment-445585826
+
+      /**
+       * Always render text using path objects (AKA outlines/curves).
+       *
+       * This setting guarantees the best quality rendering, even when using a raster paint surface
+       * (where sub-pixel path based text rendering is superior to sub-pixel text-based rendering).
+       * The downside is that text is converted to paths only, so users cannot open created vector
+       * outputs for post-processing in other applications and retain text editability.
+       *
+       * This setting also guarantees complete compatibility with the full range of formatting options available
+       * through QgsTextRenderer and QgsTextFormat, some of which may not be possible to reproduce when using
+       * a vector-based paint surface and TextFormatAlwaysText mode.
+       *
+       * A final benefit to this setting is that vector exports created using text as outlines do
+       * not require all users to have the original fonts installed in order to display the
+       * text in its original style.
+       */
+      TextFormatAlwaysOutlines,
+
+      /**
+       * Always render text as text objects.
+       *
+       * While this mode preserves text objects as text for post-processing in external vector editing applications,
+       * it can result in rendering artifacts or poor quality rendering, depending on the text format settings.
+       *
+       * Even with raster based paint devices, TextFormatAlwaysText can result in inferior rendering quality
+       * to TextFormatAlwaysOutlines.
+       *
+       * When rendering using TextFormatAlwaysText to a vector based device (e.g. PDF or SVG), care must be
+       * taken to ensure that the required fonts are available to users when opening the created files,
+       * or default fallback fonts will be used to display the output instead. (Although PDF exports MAY
+       * automatically embed some fonts when possible, depending on the user's platform).
+       */
+      TextFormatAlwaysText,
+    };
+
     /**
      * Set combination of flags that will be used for rendering.
      * \since QGIS 2.14
@@ -122,8 +167,14 @@ class CORE_EXPORT QgsRenderContext
     QPainter *painter() {return mPainter;}
 
     /**
-     * Returns the current coordinate transform for the context, or an invalid
-     * transform is no coordinate transformation is required.
+     * Returns the current coordinate transform for the context.
+     *
+     * This represents the coordinate transform required to transform a layer
+     * which is being rendered back to the CRS of the rendered map. If no coordinate
+     * transformation is required, or the render context is not associated with
+     * a map layer render, then an invalid coordinate transformation is returned.
+     *
+     * \see setCoordinateTransform()
      */
     QgsCoordinateTransform coordinateTransform() const {return mCoordTransform;}
 
@@ -171,8 +222,40 @@ class CORE_EXPORT QgsRenderContext
      */
     void setPathResolver( const QgsPathResolver &resolver ) { mPathResolver = resolver; }
 
-    const QgsRectangle &extent() const {return mExtent;}
+    /**
+     * When rendering a map layer, calling this method returns the "clipping"
+     * extent for the layer (in the layer's CRS).
+     *
+     * This extent is a "worst-case" scenario, which is guaranteed to cover the complete
+     * visible portion of the layer when it is rendered to a map. It is often larger
+     * than the actual visible portion of that layer.
+     *
+     * \warning For some layers, depending on the visible extent and the coordinate
+     * transforms involved, this extent will represent the entire globe. This method
+     * should never be used to determine the actual visible extent of a map render.
+     *
+     * \see setExtent()
+     * \see mapExtent()
+     */
+    const QgsRectangle &extent() const { return mExtent; }
 
+    /**
+     * Returns the original extent of the map being rendered.
+     *
+     * Unlike extent(), this extent is always in the final destination CRS for the map
+     * render and represents the exact bounds of the map being rendered.
+     *
+     * \see extent()
+     * \see setMapExtent()
+     * \since QGIS 3.4.8
+     */
+    QgsRectangle mapExtent() const { return mOriginalMapExtent; }
+
+    /**
+     * Returns the context's map to pixel transform, which transforms between map coordinates and device coordinates.
+     *
+     * \see setMapToPixel()
+     */
     const QgsMapToPixel &mapToPixel() const {return mMapToPixel;}
 
     /**
@@ -183,12 +266,24 @@ class CORE_EXPORT QgsRenderContext
      */
     double scaleFactor() const {return mScaleFactor;}
 
+    /**
+     * Returns TRUE if the rendering operation has been stopped and any ongoing
+     * rendering should be canceled immediately.
+     *
+     * \see setRenderingStopped()
+     */
     bool renderingStopped() const {return mRenderingStopped;}
 
+    /**
+     * Returns TRUE if rendering operations should use vector operations instead
+     * of any faster raster shortcuts.
+     *
+     * \see setForceVectorOutput()
+     */
     bool forceVectorOutput() const;
 
     /**
-     * Returns true if advanced effects such as blend modes such be used
+     * Returns TRUE if advanced effects such as blend modes such be used
      */
     bool useAdvancedEffects() const;
 
@@ -197,6 +292,11 @@ class CORE_EXPORT QgsRenderContext
      */
     void setUseAdvancedEffects( bool enabled );
 
+    /**
+     * Returns TRUE if edit markers should be drawn during the render operation.
+     *
+     * \see setDrawEditingInformation()
+     */
     bool drawEditingInformation() const;
 
     /**
@@ -207,16 +307,21 @@ class CORE_EXPORT QgsRenderContext
     double rendererScale() const {return mRendererScale;}
 
     /**
-     * Gets access to new labeling engine (may be nullptr)
+     * Gets access to new labeling engine (may be NULLPTR)
      * \note not available in Python bindings
      */
     QgsLabelingEngine *labelingEngine() const { return mLabelingEngine; } SIP_SKIP
 
+    /**
+     * Returns the color to use when rendering selected features.
+     *
+     * \see setSelectionColor()
+     */
     QColor selectionColor() const { return mSelectionColor; }
 
     /**
-     * Returns true if vector selections should be shown in the rendered map
-     * \returns true if selections should be shown
+     * Returns TRUE if vector selections should be shown in the rendered map
+     * \returns TRUE if selections should be shown
      * \see setShowSelection
      * \see selectionColor
      * \since QGIS v2.4
@@ -225,13 +330,64 @@ class CORE_EXPORT QgsRenderContext
 
     //setters
 
-    //! Sets coordinate transformation.
+    /**
+     * Sets the current coordinate transform for the context.
+     *
+     * This represents the coordinate transform required to transform the layer
+     * which is being rendered back to the CRS of the rendered map.
+     *
+     * Set to an invalid QgsCoordinateTransform to indicate that no transformation is required.
+     *
+     * \see coordinateTransform()
+     */
     void setCoordinateTransform( const QgsCoordinateTransform &t );
+
+    /**
+     * Sets the context's map to pixel transform, which transforms between map coordinates and device coordinates.
+     *
+     * \see mapToPixel()
+     */
     void setMapToPixel( const QgsMapToPixel &mtp ) {mMapToPixel = mtp;}
+
+    /**
+     * When rendering a map layer, calling this method sets the "clipping"
+     * extent for the layer (in the layer's CRS).
+     *
+     * This extent should be a "worst-case" scenario, which is guaranteed to
+     * completely cover the entire visible portion of the layer when it is rendered
+     * to the map. It may be larger than the actual visible area, but MUST contain at least the
+     * entire visible area.
+     *
+     * \see setExtent()
+     * \see setMapExtent()
+     */
     void setExtent( const QgsRectangle &extent ) {mExtent = extent;}
 
+    /**
+     * Sets the original \a extent of the map being rendered.
+     *
+     * Unlike setExtent(), this extent is always in the final destination CRS for the map
+     * render and represents the exact bounds of the map being rendered.
+     *
+     * \see mapExtent()
+     * \see setExtent()
+     * \since QGIS 3.4.8
+     */
+    void setMapExtent( const QgsRectangle &extent ) { mOriginalMapExtent = extent; }
+
+    /**
+     * Sets whether edit markers should be drawn during the render operation.
+     *
+     * \see drawEditingInformation()
+     */
     void setDrawEditingInformation( bool b );
 
+    /**
+     * Sets whether the rendering operation has been \a stopped and any ongoing
+     * rendering should be canceled immediately.
+     *
+     * \see renderingStopped()
+     */
     void setRenderingStopped( bool stopped ) {mRenderingStopped = stopped;}
 
     /**
@@ -264,6 +420,12 @@ class CORE_EXPORT QgsRenderContext
      */
     void setPainter( QPainter *p ) {mPainter = p;}
 
+    /**
+     * Sets whether rendering operations should use vector operations instead
+     * of any faster raster shortcuts.
+     *
+     * \see forceVectorOutput()
+     */
     void setForceVectorOutput( bool force );
 
     /**
@@ -271,11 +433,17 @@ class CORE_EXPORT QgsRenderContext
      * \note not available in Python bindings
      */
     void setLabelingEngine( QgsLabelingEngine *engine2 ) { mLabelingEngine = engine2; } SIP_SKIP
+
+    /**
+     * Sets the \a color to use when rendering selected features.
+     *
+     * \see selectionColor()
+     */
     void setSelectionColor( const QColor &color ) { mSelectionColor = color; }
 
     /**
      * Sets whether vector selections should be shown in the rendered map
-     * \param showSelection set to true if selections should be shown
+     * \param showSelection set to TRUE if selections should be shown
      * \see showSelection
      * \see setSelectionColor
      * \since QGIS v2.4
@@ -283,14 +451,35 @@ class CORE_EXPORT QgsRenderContext
     void setShowSelection( bool showSelection );
 
     /**
-     * Returns true if the rendering optimization (geometry simplification) can be executed
+     * Returns TRUE if the rendering optimization (geometry simplification) can be executed
      */
     bool useRenderingOptimization() const;
 
     void setUseRenderingOptimization( bool enabled );
 
-    //! Added in QGIS v2.4
+    /**
+     * Returns the simplification settings to use when rendering vector layers.
+     *
+     * The default is to use no simplification.
+     *
+     * \see setVectorSimplifyMethod()
+     * \since QGIS 2.4
+     */
     const QgsVectorSimplifyMethod &vectorSimplifyMethod() const { return mVectorSimplifyMethod; }
+
+    /**
+     * Sets the simplification setting to use when rendering vector layers.
+     *
+     * This can be used to specify simplification methods to apply during map exports and renders,
+     * e.g. to allow vector layers to be simplified to an appropriate maximum level of detail
+     * during PDF exports or to speed up layer rendering
+     *
+     * The default is to use no simplification.
+     *
+     * \see vectorSimplifyMethod()
+     *
+     * \since QGIS 2.4
+     */
     void setVectorSimplifyMethod( const QgsVectorSimplifyMethod &simplifyMethod ) { mVectorSimplifyMethod = simplifyMethod; }
 
     /**
@@ -386,6 +575,42 @@ class CORE_EXPORT QgsRenderContext
      */
     double convertMetersToMapUnits( double meters ) const;
 
+    /**
+     * Returns the text render format, which dictates how text is rendered (e.g. as paths or real text objects).
+     *
+     * \see setTextRenderFormat()
+     * \since QGIS 3.4.3
+     */
+    TextRenderFormat textRenderFormat() const
+    {
+      return mTextRenderFormat;
+    }
+
+    /**
+     * Sets the text render \a format, which dictates how text is rendered (e.g. as paths or real text objects).
+     *
+     * \see textRenderFormat()
+     * \since QGIS 3.4.3
+     */
+    void setTextRenderFormat( TextRenderFormat format )
+    {
+      mTextRenderFormat = format;
+    }
+
+    /**
+     * Returns the list of rendered feature handlers to use while rendering map layers.
+     * \see hasRenderedFeatureHandlers()
+     * \since QGIS 3.10
+     */
+    QList<QgsRenderedFeatureHandlerInterface *> renderedFeatureHandlers() const;
+
+    /**
+     * Returns TRUE if the context has any rendered feature handlers.
+     * \see renderedFeatureHandlers()
+     * \since QGIS 3.10
+     */
+    bool hasRenderedFeatureHandlers() const { return mHasRenderedFeatureHandlers; }
+
   private:
 
     Flags mFlags;
@@ -404,6 +629,7 @@ class CORE_EXPORT QgsRenderContext
     QgsDistanceArea mDistanceArea;
 
     QgsRectangle mExtent;
+    QgsRectangle mOriginalMapExtent;
 
     QgsMapToPixel mMapToPixel;
 
@@ -416,7 +642,7 @@ class CORE_EXPORT QgsRenderContext
     //! Map scale
     double mRendererScale = 1.0;
 
-    //! Newer labeling engine implementation (can be nullptr)
+    //! Newer labeling engine implementation (can be NULLPTR)
     QgsLabelingEngine *mLabelingEngine = nullptr;
 
     //! Color used for features that are marked as selected
@@ -442,6 +668,10 @@ class CORE_EXPORT QgsRenderContext
 
     QgsPathResolver mPathResolver;
 
+    TextRenderFormat mTextRenderFormat = TextFormatAlwaysOutlines;
+    QList< QgsRenderedFeatureHandlerInterface * > mRenderedFeatureHandlers;
+    bool mHasRenderedFeatureHandlers = false;
+
 #ifdef QGISDEBUG
     bool mHasTransformContext = false;
 #endif
